#############################################################
#
#  Author: Sebastian Maurice, PhD
#  Copyright by Sebastian Maurice 2018
#  All rights reserved.
#  Email: Sebastian.maurice@gmail.com
#
#############################################################

import imp
import json, urllib
import requests
import maads
import urllib.request
import os


#maads = imp.load_source('maads','sendfiles.py')

    
def dotraining(thefile,username,passw,autofeature,removeoutliers,hasseasonality,dependentvariable,company,email,url,summer='6,7,8',winter='12,1,2,3',shoulder='4,5,9,10,11',trainingpercentage=75,retrainingdays=0,retraindeploy=0,shuffle=0):
#   rmsg=maads.uploadcsvfortraining(thefile,username,passw,autofeature,removeoutliers,hasseasonality,dependentvariable,company,email,url,summer,winter,shoulder,trainingpercentage)
   #print(rmsg)
   if autofeature != -1 and autofeature != 1 and autofeature != 0:
      return "ERROR: Incorrect autofeature value: 1,0,-1"
   if removeoutliers != 1 and removeoutliers != 0:
      return "ERROR: Incorrect removeoutlier value value: 1,0"
   if hasseasonality != 1 and hasseasonality != 0:
      return "ERROR: Incorrect hasseasonality value value: 1,0"
      
   if trainingpercentage<=40 or trainingpercentage>=80:
      return "ERROR: Incorrect training percentage value: Must be between 40 and 80"
   if hasseasonality:
      if len(summer)==0 or len(winter)==0 or len(shoulder)==0:
         return "ERROR: Summer, Winter and Shoulder must contain month values."
      if len(summer)>0:
         ss=summer.split(",")
         for i in ss:
            if int(i)<1 or int(i)>12:
               return "ERROR: Summer months: Months must be between 1 and 12."
      if len(winter)>0:
         ss=winter.split(",")
         for i in ss:
            if int(i)<1 or int(i)>12:
               return "ERROR: Winter months: Months must be between 1 and 12."
      if len(shoulder)>0:
         ss=shoulder.split(",")
         for i in ss:
            if int(i)<1 or int(i)>12:
               return "ERROR: Shoulder months: Months must be between 1 and 12."

   if  retrainingdays<0:
      return "ERROR: Invalid retraining day."
   else:
      retrainingdays=int(retrainingdays)

   if  retraindeploy<0 or retraindeploy>1:
      return "ERROR: Invalid retraining deploy: must be 0 or 1."
   else:
      retraindeploy=int(retraindeploy)
      
   rmsg=maads.uploadcsvfortraining(thefile,username,passw,autofeature,removeoutliers,hasseasonality,dependentvariable,company,email,url,summer,winter,shoulder,trainingpercentage,retrainingdays,retraindeploy,shuffle)
   rms=rmsg.split(":")[0]
   rms=rms.lstrip('\n')
   rms=rms.rstrip('\n')
   rms=rms.strip()
   if rms.lower()=='error':
        #print(thedata)
      return rmsg
    
   if rmsg.find("FEATURECOMPLETE:")!=-1:
      fn=rmsg[18:]
      url = "%s" % (fn)
      #print(url)
      base=os.path.basename(thefile)
      floc=os.path.splitext(base)[0]
      if len(os.path.dirname(thefile))>0:
          localname=os.path.dirname(thefile)+ "/" +floc +"_features.csv"
      else:
          localname=floc +"_features.csv"
          
      #print(localname)
      urllib.request.urlretrieve(url, localname)
      return "Feature file retrieved:" + localname
   
   pkeylist=rmsg.splitlines()
   #print(pkeylist)
   pkeyname=pkeylist[3]
   #print(pkeyname)
   
   pkey=pkeyname[5:]
   #print(pkey)
   base=os.path.basename(thefile)
   floc=os.path.splitext(base)[0]
   if len(os.path.dirname(thefile))>0:
      localpdf=os.path.dirname(thefile)+ "/" +floc +"_REPORT.pdf"
      localpred=os.path.dirname(thefile)+ "/" +floc +"_PREDICTION.csv"
      localfeat=os.path.dirname(thefile)+ "/" +floc +"_FEATURES.csv"
   else:
      localpdf=floc +"_REPORT.pdf"
      localpred=floc +"_PREDICTION.csv"
      localfeat=floc +"_FEATURES.csv"
      
   a=url.split("/")
   a=a[:-1]
   b='/'.join(a)
   
   try:   
      remotepdf="%s.pdf" % (pkey)
      url = "%s/pdfreports/%s" % (b,remotepdf)
      #print(url)
      urllib.request.urlretrieve(url, localpdf)
   except Exception as e:
      a=1
      print("Cannot download remotefile: %s TO %s" % (url,localpdf))
   
   #Prediction Data
   try:
      remotepred="%s.csv" % (pkey)
      url = "%s/csvuploads/%s" % (b,remotepred)
      #print(url)
      urllib.request.urlretrieve(url, localpred)
   except Exception as e:
      a=1
      print("Cannot download remotefile: %s TO %s" % (url,localpred))


   if autofeature==1:
      try:
         remotefeat="%s_.csv" % (pkey)
         url = "%s/autofeatures/%s" % (b,remotefeat)
         urllib.request.urlretrieve(url, localfeat)
      except Exception as e:
         a=1
         print("Cannot download remotefile: %s TO %s" % (url,localfeat))
      
   return rmsg

def dopredictions(attr,pkey,thefile,username,passw,company,email,url):
   rmsg=maads.getpredictions(attr,pkey,thefile,username,passw,company,email,url)
   #print(rmsg)
   return rmsg





#############################################################
#
#  Author: Sebastian Maurice, PhD
#  Copyright by Sebastian Maurice 2018
#  All rights reserved.
#  Email: Sebastian.maurice@gmail.com
#
#############################################################

import json, urllib
import requests
import csv
import os
import imp
import re
import urllib.request
import asyncio
import validators

    
async def tcp_echo_client(message, loop,host,port):
    reader, writer = await asyncio.open_connection(host, port,
                                                   loop=loop)

    mystr=str.encode(message)
    writer.write(mystr)
    data = await reader.read(2096)
    prediction=("%s" % (data.decode()))
    writer.close()
    
    return prediction

def hyperpredictions(host,port,username,password,company,email,pkey,theinputdata):
    theinputdata=theinputdata.replace(",",":")
    value="%s,%s,%s,%s,%s,[%s]" % (username,password,company,email,pkey,theinputdata)
    loop = asyncio.get_event_loop()
    val=loop.run_until_complete(tcp_echo_client(value, loop,host,port))
    return val

def returndata(buffer,label):
      #print("LABEL: %s" % (label))
      if label=='PKEY:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]
      elif label=='ALGO0:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         #print(listvalues)
         val=[s for s in listvalues if label in s]
         #print(val)
         rval=val[0].split(':')[1]
      elif label=='ACCURACY0:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]
      elif label=='SEASON0:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]         

      elif label=='ALGO1:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         #print(listvalues)
         val=[s for s in listvalues if label in s]
         #print(val)
         rval=val[0].split(':')[1]
      elif label=='ACCURACY1:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]
      elif label=='SEASON1:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]         
      elif label=='ALGO2:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         #print(listvalues)
         val=[s for s in listvalues if label in s]
         #print(val)
         rval=val[0].split(':')[1]
      elif label=='ACCURACY2:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]
      elif label=='SEASON2:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]         
      elif label=='ALGO3:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         #print(listvalues)
         val=[s for s in listvalues if label in s]
         #print(val)
         rval=val[0].split(':')[1]
      elif label=='ACCURACY3:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]
      elif label=='SEASON3:':
         val=""
         pattern = re.compile('\s*[,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         val=[s for s in listvalues if label in s]      
         rval=val[0].split(':')[1]         
         
      elif label=='DATA:':
         val=""
         pattern = re.compile('\s*[:,\n]\s*')
         fixed = pattern.sub(', ', buffer)
         listvalues=fixed.split(', ')
         #print(listvalues)
         fdate=listvalues[1]
         inp=listvalues[2]
         pred=float(listvalues[3])
         acc=float(listvalues[4])
         rval=[fdate,inp,pred,acc]
      else:
         return "%s not found" % (label)
          
      return rval

def retraining(pkey,thefile,username,passw,autofeature,removeoutliers,hasseasonality,dependentvariable,company,email,url,summer,winter,shoulder,trainingpercentage,retrainingdays,retraindeploy):

   rn=0
   tstr=''
   
   with open(thefile, 'r') as f:
     reader = csv.reader(f)
     for row in reader:
       row = ",".join(row)
       tstr = tstr + str(row) + '\n'
       rn=rn+1
       
   head, fname = os.path.split(thefile)
   print("Please wait...training can take several minutes.")
   
   files = {'file': tstr,
    'mode':-1,        
    'type':'CSV',
    'filename':fname,
    'username': username,
    'password': passw,
    'rowcount': rn,
    'autofeature': autofeature,
    'removeoutliers': removeoutliers,
    'hasseasonality': hasseasonality,
    'company': company,
    'email': email,            
    'dependentvariable': dependentvariable,
    'title':'File Upload for Training',
    'summer':summer,
    'winter':winter,
    'shoulder':shoulder,
    'trainingpercentage':trainingpercentage,
    'retrainingdays':retrainingdays,
    'retraindeploy':retraindeploy,
    'pkey':pkey
            
   }

   #print(files)
   r = requests.post(url, files)
   msg = r.text
   #print ("Message %s" % (msg))
   
   return msg

def uploadcsvfortraining(thefile,username,passw,autofeature,removeoutliers,hasseasonality,dependentvariable,company,email,url,summer,winter,shoulder,trainingpercentage,retrainingdays,retraindeploy,shuffle):

   rn=0
   tstr=''
   
   with open(thefile, 'r') as f:
     reader = csv.reader(f)
     for row in reader:
       row = ",".join(row)
       tstr = tstr + str(row) + '\n'
       rn=rn+1
       
   head, fname = os.path.split(thefile)
   print("Please wait...training can take several minutes.")
   
   files = {'file': tstr,
    'mode':0,        
    'type':'CSV',
    'filename':fname,
    'username': username,
    'password': passw,
    'rowcount': rn,
    'autofeature': autofeature,
    'removeoutliers': removeoutliers,
    'hasseasonality': hasseasonality,
    'company': company,
    'email': email,            
    'dependentvariable': dependentvariable,
    'title':'File Upload for Training',
    'summer':summer,
    'winter':winter,
    'shoulder':shoulder,
    'trainingpercentage':trainingpercentage,
    'retrainingdays':retrainingdays,
    'retraindeploy':retraindeploy,
    'shuffle':shuffle
            
   }

   #print(files)
   r = requests.post(url, files)
   msg = r.text
   #print ("Message %s" % (msg))
   
   return msg

def getpredictions(attr,pkey,thefile,username,passw,company,email,url):

   rn=0
   tstr=''

   
   if attr==0:
      tstr=thefile
         
      files = {'file': tstr,
        'mode':1,        
        'type':'CSV',
        'pkey':pkey,            
        'username': username,
        'password': passw,
    #'rowcount': rn,
    #'autofeature': autofeature,
    #'removeoutliers': removeoutliers,
    #'hasseasonality': hasseasonality,
       'company': company,
       'email': email,            
    #'dependentvariable': dependentvariable,
       'title':'Do Predictions'
      }

   #print(files)
      r = requests.post(url, files)
      msg = r.text
   #print ("Message %s" % (msg))
   
      return msg


def dolistkeys(username,passw,company,email,url):

   rn=0
   tstr=''

   
   files = {
      'mode':2,        
      'type':'CSV',        
      'username': username,
      'password': passw,
      'company': company,
      'email': email,              
     'title':'Do List keys'
   }

   #print(files)
   r = requests.post(url, files)
   msg = r.text
   #print ("Message %s" % (msg))
   
   return msg

def dolistkeyswithkey(username,passw,company,email,pkey,url):

   rn=0
   tstr=''

   
   files = {
      'mode':3,
      'pkey':pkey,        
      'type':'CSV',        
      'username': username,
      'password': passw,
      'company': company,
      'email': email,              
     'title':'Do List keys with Key'
   }

   #print(files)
   r = requests.post(url, files)
   msg = r.text
   #print ("Message %s" % (msg))
   
   return msg

def dodeletewithkey(username,passw,company,email,pkey,url):

   rn=0
   tstr=''

   
   files = {
      'mode':4,
      'pkey':pkey,        
      'type':'CSV',        
      'username': username,
      'password': passw,
      'company': company,
      'email': email,              
     'title':'Do Delete with Key'
   }

   #print(files)
   r = requests.post(url, files)
   msg = r.text
   #print ("Message %s" % (msg))
   
   return msg



def getpicklezip(username,passw,company,email,pkey,url,localfolder):

    url = "%s/prodfiles/%s_DEPLOYTOPROD.zip" % (url,pkey)
    localname="%s/%s_DEPLOYTOPROD.zip" % (localfolder,pkey)
    urllib.request.urlretrieve(url, localname)
    #print(url)
    return "file retrieved"


def sendpicklezip(username,passw,company,email,pkey,url,localname):
    bn=os.path.basename(localname)
    data = {'mode':'uploads', 'username':username, 'password':passw,'company':company,'email':email,'pkey':pkey}
    
    files = {'file': open(localname, 'rb')}
    r = requests.post(url, data=data, files=files)
    return r.text
    
def deploytoprod(username,passw,company,email,pkey,url,localname='',ftpserver='',ftpuser='',ftppass=''):

    data = {'mode':'deploy', 'username':username, 'password':passw,'company':company,'email':email,'localname':localname,'pkey':pkey,'ftpserver':ftpserver,'ftpuser':ftpuser,'ftppass':ftppass}

    #print(prodserverurl)

    
    if len(localname)>0:
        bn=os.path.basename(localname)
        data = {'mode':'deploy', 'username':username, 'password':passw,'company':company,'email':email,'localname':bn,'pkey':pkey,'ftpserver':ftpserver,'ftpuser':ftpuser,'ftppass':ftppass}        
        files = {'file': open(localname, 'rb')}
        r = requests.post(url, data=data, files=files)
    else:
        bn="%s_DEPLOYTOPROD.zip" % (pkey)
        data = {'mode':'deploy', 'username':username, 'password':passw,'company':company,'email':email,'localname':localname,'pkey':pkey,'ftpserver':ftpserver,'ftpuser':ftpuser,'ftppass':ftppass}                
        r = requests.post(url, data=data)
 #   print(r.text)    
    return r.text

def nlp(username,passw,company,email,buffer,url,detail=20):
    isurl=0

    if validators.url(buffer):
        isurl=1
    else:
        isurl=0
        
    if os.path.isfile(buffer):  #pdf
        filename, file_extension = os.path.splitext(buffer)
        flower=file_extension.lower()
        bn=os.path.basename(buffer)
        if flower=='.pdf':         
           files = {'file': open(buffer, 'rb')}
        elif flower=='.txt':
           files = {'file': open(buffer, 'r')}               
        data = {'mode':'nlp1', 'username':username, 'password':passw,'company':company,'email':email,'localname':bn,'fvalue': detail}
        r = requests.post(url, data=data, files=files)
    elif isurl==1:  #url
        data = {'mode':'nlp2', 'username':username, 'password':passw,'company':company,'email':email,'localname':buffer,'fvalue': detail}
        r = requests.post(url, data=data)
    else: #paste text
        data = {'mode':'nlp3', 'username':username, 'password':passw,'company':company,'email':email,'localname':buffer,'fvalue': detail}
        r = requests.post(url, data=data)
    return r.text

#csvfile,iscategory,maads_rest_url,trainingpercentage,retrainingdays,retraindeploy
def nlpclassify(username,passw,company,email,thefile,iscategory,maads_rest_url,trainingpercentage=75,retrainingdays=0,retraindeploy=0):

    tstr=''
    rn=0
    with open(thefile, 'r') as f:
     reader = csv.reader(f)
     for row in reader:
       row = ",".join(row)
       tstr = tstr + str(row) + '\n'
       rn=rn+1
    base=os.path.basename(thefile)
    filename=os.path.splitext(base)[0]
    
    files = {'file': tstr,
        'mode':'nlpclassify',        
        'type':'CSV',
        'iscategory':iscategory,            
        'username': username,
        'password': passw,
        'trainingpercentage': trainingpercentage,
        'retrainingdays': retrainingdays,
        'retraindeploy': retraindeploy,
        'company': company,
        'email': email,
        'filename':filename,      
    #'dependentvariable': dependentvariable,
        'title':'Do NLP Classify'
      }

    r = requests.post(maads_rest_url, files)
    msg = r.text
   
    return msg
                
#getpicklezip('demouser','demouser0828','OTICS','sebastian.maurice@otics.ml','demouser_acnstocksdatatest_csv','http://www.otics.ca/maadsweb','c:/maads')



import setuptools

from os import path
#this_directory = path.abspath(path.dirname(__file__))
#with open(path.join(this_directory, 'README.md')) as f:
with open("README.md", "r") as f:
    long_description = f.read()
	
setuptools.setup(
    name='maads',
    version='3.14.36',
    description='Multi-Agent Accelerator for Data Science',
    license='OTICS Advanced Analytics Inc.',
    packages=['maads'],
    author='Sebastian Maurice',
    author_email='sebastian.maurice@otics.ca',
    keywords=['multi-agent, data science, machine learning, automl,auto-ml,artificial intelligence', 'predictive analytics', 'advanced analytics'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/smaurice101/acnsmauricedsmas'
)

